#!/usr/bin/env python
"""
--------------------------------------------------------------------------------
Copyright (C) 2005 Leland McInnes <lmcinnes@uwo.ca>

Copyright (C) Everaldo (http://www.everaldo.com) for all icons.
    Used with permission.
   
Released under the GPL, version 2. Except the artwork, which is released
pursuant to the restrictions imposed by their authors.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.   

--------------------------------------------------------------------------------
This was built on python 2.4, on a linux 2.6.10-1.770_FC2 glibc 2.3.3
x86 machine. Just run this script from its final location on you disk to
start the desklet. See README for details.

*** DO NOT edit this file for changing the configuration; edit instead
`config.txt' in this directory ***

--------------------------------------------------------------------------------
"""

import adesklets
import statgrab

from os import getenv, system
from os.path import join, dirname
from math import log
from random import random
from re import sub, split
from string import strip

class Config(adesklets.ConfigFile):
    """
 This is SystemMonitor.py desklet configuration file.  Yes, it is complicated.
    
 There is one main key "meters" which has a list of meters to include.  Each 
 meter is a 2-tuple of the meter type, and the configuration for the meter.  
 Presently available meter types are: CPUMeter, MemoryMeter, NetworkMeter,  
 DiskIOMeter, DiskSpaceMeter, TemperatureMeter, and BatteryMeter.  
 Those should all be self explanatory as to what they measure.
 
 CPU, Memory, Swap and DiskSpace meters are straightforward, just provide the 
 images, font type and size, and padding internal to the meter.  Optionally
 you can add an update speed, to specify that this meter shoould check for
 updates less often.  Update speed should be set to the number of seconds
 between updates.
  
 Network and DiskIO meters require additional information: NetworkMeters 
 require an interface name.  This must match up with a network interface, 
 so names should be of the sort eth0, ppp0, etc.  If you have multiple 
 interfaces, just add multiple network meters, one for each interface.  You 
 must also specify maximum download and upload speeds in kilobytes per second.  
 The scale is logarithmic, so if you  overshoot it doesn't matter much - 
 undershooting will make the meter max out all the time - be generous.  DiskIO 
 is similar, except you now need to supply the disk name (hda, hdb, hdc etc.) 
 and max read and write speeds.  Again, better not to underestimate those 
 figures.
 
 TemperatureMeter and BatteryMeter require ACPI support.  If you don't have
 ACPI support these meters will simply display as blank, so just comment them
 out of the configuration.  For TemperatureMeter you must supply the file
 to read for temperatures (/proc/acpi/thermal_zone/THM/temperature on my 
 system), and the max_temp (in degrees celsius) to provide some scale.
 BatteryMeter requires you to provide two acpi files, the info and state
 files.  You also need to provide a label, as multiple batteries are 
 supported (simply add multiple BatteryMeters with multiple labels and
 different info and state files).
 
 For meters that have multiple meters/krells (Network, DiskIO, DiskSpace)
 you can specify a list of images to use for the krells.  The list must be
 the same length as the number of meters/krells to be displayed (2 in the
 case of Network and DiskIO, variable depending on your patition config
 in the case of DiskSpace).
 
 There are also keys for the background image set (a set of three images,
 given as a list), the background colour (what colour the background of the
 meter blocks is), and the text colour.  You can also specify a global
 update speed for all meters.  If the desklet is using too much CPU for you
 then throttle back the update speed to update every 5 seconds or so, which will
 drastically decrease CPU use.
 
 An alternate configuration could look like:
 {"meters" : [
              ("CPUMeter",
               {
                "icon" : "images/icons/cpu.png",
                "trough" : "images/shared/trough.png",
                "krell" : "images/shared/krell_blue.png",
                "meter font name" : "Vera",
                "meter font size" : 11,
                "vertical padding" : 4,
                "horizontal padding" : 12
               }
              ),
              ("DiskSpaceMeter",
               {
                "icon" : "images/icons/disk.png",
                "trough" : "images/shared/trough_small.png",
                "krell" : "images/shared/krell_green_small.png",
                "meter font name" : "Vera",
                "meter font size" : 7,
                "vertical padding" : 2,
                "horizontal padding" : 12
               }
              ),
              ("NetworkMeter",
               {
                "interface name" : "ppp0",
                "max down speed" : 8,
                "max up speed" : 5,
                "icon" : "images/icons/network-ppp.png",
                "trough" : "images/shared/trough.png",
                "krell" : ["images/shared/krell_green_small.png",
                           "images/shared/krell_red_small.png"],
                "meter font name" : "Vera",
                "meter font size" : 7,
                "vertical padding" : 4,
                "horizontal padding" : 8
               }
              )              
             ],
  "update_speed" : 5
 }               
    """
    cfg_default = {
                   "meters" : [
                               ("CPUMeter",
                                {
                                 "icon" : "images/icons/cpu.png",
                                 "trough" : "images/shared/trough.png",
                                 "krell" : "images/shared/krell.png",
                                 "meter font name" : "VeraBd",
                                 "meter font size" : 8,
                                 "vertical padding" : 8,
                                 "horizontal padding" : 8
                                }
                               ),
                               ("MemoryMeter",
                                {
                                 "icon" : "images/icons/memory.png",
                                 "trough" : "images/shared/trough.png",
                                 "krell" : "images/shared/krell.png",
                                 "meter font name" : "VeraBd",
                                 "meter font size" : 8,
                                 "vertical padding" : 8,
                                 "horizontal padding" : 8,
                                 "update speed" : 10
                                }
                               ),
                               ("SwapMeter",
                                {
                                 "icon" : "images/icons/swap.png",
                                 "trough" : "images/shared/trough.png",
                                 "krell" : "images/shared/krell_blue.png",
                                 "meter font name" : "VeraBd",
                                 "meter font size" : 8,
                                 "vertical padding" : 8,
                                 "horizontal padding" : 8,
                                 "update speed" : 30
                                }
                               ),
                               ("NetworkMeter",
                                {
                                 "interface name" : "eth0",
                                 "max down speed" : 400,
                                 "max up speed" : 80,
                                 "icon" : "images/icons/network.png",
                                 "trough" : "images/shared/trough_small.png",
                                 "krell" : ["images/shared/krell_green_small.png",
                                            "images/shared/krell_red_small.png"],
                                 "meter font name" : "Vera",
                                 "meter font size" : 7,
                                 "vertical padding" : 4,
                                 "horizontal padding" : 8,
                                 "update speed" : 2
                                }
                               ),
                               ("DiskIOMeter",
                                {
                                 "disk name" : "hda",
                                 "max read speed" : 10240,
                                 "max write speed" : 10240,
                                 "icon" : "images/icons/disk.png",
                                 "trough" : "images/shared/trough_small.png",
                                 "krell" : ["images/shared/krell_green_small.png",
                                            "images/shared/krell_red_small.png"],
                                 "meter font name" : "Vera",
                                 "meter font size" : 7,
                                 "vertical padding" : 4,
                                 "horizontal padding" : 8,
                                 "update speed" : 2
                                }
                               ),
                               ("DiskSpaceMeter",
                                {
                                 "icon" : "images/icons/diskfree.png",
                                 "trough" : "images/shared/trough_small.png",
                                 "krell" : "images/shared/krell_blue_small.png",
                                 "meter font name" : "Vera",
                                 "meter font size" : 6,
                                 "vertical padding" : 2,
                                 "horizontal padding" : 8,
                                 "update speed" : 60
                                }
                               ),
                               ("TemperatureMeter",
                                {
                                 "icon" : "images/icons/temperature.png",
                                 "trough" : "images/shared/trough.png",
                                 "krell" : "images/shared/krell_red.png",
                                 "meter font name" : "VeraBd",
                                 "meter font size" : 8,
                                 "vertical padding" : 8,
                                 "horizontal padding" : 8,
                                 "file" : "/proc/acpi/thermal_zone/THM/temperature",
                                 "max_temp" : 100.0,
                                 "update_speed" : 30
                                }
                               ),
                               ("BatteryMeter",
                                {
                                 "icon" : "images/icons/battery.png",
                                 "trough" : "images/shared/trough.png",
                                 "krell" : "images/shared/krell_green.png",
                                 "meter font name" : "VeraBd",
                                 "meter font size" : 8,
                                 "vertical padding" : 8,
                                 "horizontal padding" : 8,
                                 "label" : "",
                                 "state_file" : "/proc/acpi/battery/BAT0/state",
                                 "info_file" : "/proc/acpi/battery/BAT0/info",
                                 "update_speed" : 10
                                }
                               )                                                                
                              ],
                   "background images" : ["images/shared/bg_top.png",
                                          "images/shared/bg_middle.png",
                                          "images/shared/bg_bottom.png"],
                   "background colour" : (210, 210, 210, 130),
                   "text colour" : (0, 0, 0, 200),
                   "update speed" : 1
                  }

    def __init__(self, identifier, filename):
        adesklets.ConfigFile.__init__(self, identifier, filename)

# This is  where all the ahrd work gets done is terms of creating, drawing,
# and updating meters and associated text occurs.  Subclasses should mostly
# be able to just inherit this and proceed to only override "getMeterValues()"
class AbstractMeter (object):
    """
    Base class for all meters - allows for an icon and some number of
    krells/meters displaying data, along with a textual description
    of the data for each meter.  Specifics of data collected, and how
    it should be interpreted/displayed is handled by subclasses
    """ 
    
    def __init__(self):
        """
        Create a new Abstract Meter
        """
        self.icon = None
        self.buffer = None
        self.krell = []
        self.trough = []
        self.base_text = []
        self.value_text = []
        self.update_speed = 0
        
        self._x_offset = 0
        self._y_offset = 0
        self._icon_x_offset = 0
        self._icon_y_offset = 0
        self._icon_width = 0
        self._icon_height = 0
        self._buffer_x_offset = 0
        self._buffer_y_offset = [0]
        self._krell_width = 0
        self._krell_height = 0
        self._text_width = [0]
        self._text_height = 0
        self._horizontal_padding = 0
        self._vertical_padding = 0
        self._bounds = [0, 0, 0, 0]
        self._meter_font = None
        self._macro = [0]
        self._macro_krell = [0]
        self._macro_text = [0]
        self._update_due = 0
        self._background_colour = (210, 210, 210, 130)
        self._text_colour = (0, 0, 0, 200)
        self._last_value = [-1]

    # Very little work done here, mostly just establishing correctness 
    # of the dictionary, and making the required assignments.  This is 
    # only a separate procedure for tidiness sake
    def _create_fonts(self, config_dictionary):
        """
        Create fonts as specified in a configuration dictionary
        """
#        assert type(config_dictionary) is dict
#        assert "meter font size" in config_dictionary.keys() and \
#            type(config_dictionary["meter font size"]) is int
#        assert "meter font name" in config_dictionary.keys() and \
#            type(config_dictionary["meter font name"]) is str
        
        self._meter_font_size = config_dictionary["meter font size"]
        self._meter_font_name = config_dictionary["meter font name"]        
        self._meter_font = adesklets.load_font(self._meter_font_name + "/" + \
                                                   str(self._meter_font_size))

    # Very little work done here, mostly just establishing correctness 
    # of the dictionary, and making the required assignments.  This is 
    # only a separate procedure for tidiness sake       
    def _create_icon(self, basedir, config_dictionary):
        """
        Create the icon pixmap specified by a configuration dictionary
        """
#        assert type(config_dictionary) is dict
#        assert "icon" in config_dictionary.keys()
#        assert type(basedir) is str and len(basedir) >= 1
        
        # create icon and set width and height values
        self.icon = adesklets.load_image(join(basedir,config_dictionary["icon"]))

        adesklets.context_set_image(self.icon)
        self._icon_width = adesklets.image_get_width()
        self._icon_height = adesklets.image_get_height()   
        
    # Very little work done here, mostly just establishing correctness 
    # of the dictionary, and making the required assignments.  This is 
    # only a separate procedure for tidiness sake        
    def _create_meters(self, basedir, config_dictionary):
        """
        Create pixmaps and base text for each meter according to a 
        configuration dictionary
        """
#        assert type(config_dictionary) is dict
#        assert "krell" in config_dictionary.keys()
#        assert "trough" in config_dictionary.keys()
#        assert "base text" in config_dictionary.keys()
#        assert "value text length" in config_dictionary.keys()
#        assert type(config_dictionary["base text"]) is tuple and \
#                   len(config_dictionary["base text"]) == self._number_of_meters
#        assert type(config_dictionary["value text length"]) is tuple and \
#                   len(config_dictionary["base text"]) == self._number_of_meters
#        assert self._meter_font is not None
#        assert type(basedir) is str and len(basedir) >= 1     
        
        adesklets.context_set_font(self._meter_font)
        
        # create image sets and text for each meter, and establish 
        # base text and text widths        
        for index in range(self._number_of_meters):
            self.trough[index] = adesklets.load_image(join(basedir, 
                                                      config_dictionary["trough"]))
            
            # Allow a list of krell images, but accept a single string if used                                          
            if (type(config_dictionary["krell"]) is str):
                self.krell[index] = adesklets.load_image(join(basedir,
                                                         config_dictionary["krell"]))
            elif (type(config_dictionary["krell"]) is list):
                try:
                    self.krell[index] = adesklets.load_image(join(basedir,
                                                             config_dictionary["krell"][index]))
                except IndexError:
                    raise IndexError("list must have exactly as many entries as meters")
            else:
                raise TypeError("krell key must provide a single path, or list of paths")
            
            self.base_text[index] = config_dictionary["base text"][index]
            self.value_text[index] = "_" * config_dictionary["value text length"][index]
            self._text_width[index], self._text_height = adesklets.get_text_size(self.base_text[index] 
                                                                                 + " " + self.value_text[index])         
        
        # set size for common image used for meters
        adesklets.context_set_image(self.krell[0])
        self._krell_width = adesklets.image_get_width()
        self._krell_height = adesklets.image_get_height()
        
    # Collect all positioning work into a single procedure
    def setOffset(self, x_offset, y_offset):
        """
        Position all the elements relative to a abse x and y offset for the whole meter
        """
        self._x_offset = x_offset
        self._y_offset = y_offset
        
        # adjust offsets according to placement and padding
        self._icon_x_offset = self._x_offset + self._horizontal_padding
        self._buffer_x_offset = self._icon_x_offset + self._icon_width + \
                                   self._horizontal_padding                                  

        # stack krells and text
        tmp_height = 8
        
        for index in range(self._number_of_meters):
            self._buffer_y_offset[index] = self._y_offset + tmp_height
            tmp_height += self._krell_height + self._vertical_padding
            tmp_height += self._text_height + 8

        self._buffer_width = max(self._krell_width, max(self._text_width))
        self._buffer_height = (self._krell_height + self._text_height + 2 * self._vertical_padding)
            
        # define bounds - useful for all manner of things (mostly unimplemented)
        self._bounds[0] = self._x_offset
        self._bounds[1] = self._y_offset
        self._bounds[2] = self._buffer_x_offset + \
                              max(self._krell_width, max(self._text_width)) + \
                                  self._horizontal_padding
                                  
        self._bounds[3] = self._y_offset + tmp_height
        
        # now set the icon vertical offset to center the 
        # icon vertically (height not set previously)
        self._icon_y_offset = self._y_offset + \
                                  int((self.getHeight() - self._icon_height)/2.0)
        
    # This is where there work of creating and positioning meters is done.  
    # Minor tasks are grouped logically and farmed out to sub procedures 
    # which in turn further verify the correctness of the configuration 
    # dictionary for their private tasks.
    def create(self, location, basedir, config_dictionary):
        """
        Instantiate the meter with a given configuration
        """
#        assert len(location) == 2 and type(location[0]) is int and \
#            type(location[1]) is int
#        assert type(config_dictionary) is dict
#        assert "horizontal padding" in config_dictionary.keys()
#        assert "vertical padding" in config_dictionary.keys()
#        assert "background colour" in config_dictionary.keys()
#        assert type(basedir) is str and len(basedir) >= 1
              
        # determine how many meters to associate with the icon
        if (config_dictionary.has_key("number of meters")):
            self._number_of_meters = config_dictionary["number of meters"]
        else:
            self._number_of_meters = 1
            
        self._background_colour = config_dictionary["background colour"]
        self._text_colour = config_dictionary["text colour"]
        
        if (config_dictionary.has_key("update speed")):
            self.update_speed = config_dictionary["update speed"]
            
        # create arrays for meter and text offsets, and base strings for text
        self.krell = [0] * self._number_of_meters
        self.trough = [0] * self._number_of_meters
        self._buffer_y_offset = [0] * self._number_of_meters
        self._text_width = [0] * self._number_of_meters
        self._last_value = [-1] * self._number_of_meters
        self.base_text = [""] * self._number_of_meters
        self.value_text = [""] * self._number_of_meters
        self._macro = [0] * self._number_of_meters
        self._macro_krell = map(lambda x: "krell" + str(random()), range(self._number_of_meters))
        self._macro_text = map(lambda x: "text" + str(random()), range(self._number_of_meters))
        
        # set up fonts, icon, and meters
        self._create_fonts(config_dictionary)
        self._create_icon(basedir, config_dictionary)
        self._create_meters(basedir, config_dictionary)
        
        # establish padding
        self._horizontal_padding = config_dictionary["horizontal padding"]
        self._vertical_padding = config_dictionary["vertical padding"]

        # position all elements (icon, krell, text) with respect to base offsets
        self.setOffset(location[0], location[1])
        
        # create drawing buffer for meters
        self.buffer = adesklets.create_image(self._buffer_width, self._buffer_height)
                   

    # Generate a new meter display creating icons, trough, krell and text for
    # the first time.  After this all image changes will occur via "update()"
    def display(self):
        """Draw the meter to screen"""
#        assert self.icon is not None
#        assert self.krell is not None
#        assert type(self.base_text) is list and \
#            len(self.base_text) == self._number_of_meters
#        assert type(self.value_text) is list and \
#            len(self.value_text) == self._number_of_meters
        
        #draw background
        adesklets.context_set_blend(False)
        adesklets.context_set_color(*self._background_colour)
        adesklets.image_fill_rectangle(self._x_offset, self._y_offset, 
                                       self.getWidth(), self.getHeight())
        adesklets.context_set_blend(True)
        
        # draw meter icon
        adesklets.blend_image_onto_image(self.icon, 1, 0, 0, 
                                         self._icon_width, self._icon_height, 
                                         self._icon_x_offset, self._icon_y_offset, 
                                         self._icon_width, self._icon_height)
 
        for index in range(self._number_of_meters):
            
            adesklets.set(self._macro_krell[index], 0)
            adesklets.set(self._macro_text[index], self.base_text[index] + " " + self.value_text[index])
            
            adesklets.start_recording()
            
            adesklets.context_set_image(self.buffer)
            
            # draw background
            adesklets.context_set_color(*self._background_colour)
            adesklets.image_fill_rectangle(0, 0, 
                                           self._buffer_width, self._buffer_height)
            adesklets.context_set_blend(True)
            
            # draw trough and krell
            adesklets.blend_image_onto_image(self.trough[index], 1, 0, 0, 
                                             self._krell_width, self._krell_height, 
                                             0, 0, 
                                             self._krell_width, self._krell_height)

            adesklets.blend_image_onto_image(self.krell[index], 0, 0, 0, 
                                             '$' + self._macro_krell[index], self._krell_height, 
                                             0, 0, 
                                             '$' + self._macro_krell[index], self._krell_height)
                                             
            # render text
            adesklets.context_set_color(*self._text_colour)
            adesklets.context_set_font(self._meter_font)
            adesklets.text_draw(0, self._krell_height + self._vertical_padding, '$' + self._macro_text[index])
            
            # render buffer to base image
            adesklets.context_set_image(0)
            adesklets.context_set_blend(False)
            adesklets.blend_image_onto_image(self.buffer,1,
                                             0,0,
                                             self._buffer_width, self._buffer_height,
                                             self._buffer_x_offset,self._buffer_y_offset[index],
                                             self._buffer_width, self._buffer_height)
            
            self._macro[index] = adesklets.stop_recording()
            adesklets.play_set_abort_on_events(False)
            
        self.update()
     
    # Virtual to be overidden by subclasses
    def getMeterValues(self):
        """
        Virtual method to return krell and text values
           returns a 2-tuple of: 
               a float between 0 and 1.0 for krell length/scale
               a string describing the value, not longer than 6 chars
        """
        return

    # Determine whether an update is required or not.
    def requireUpdate(self, update_speed):
        if (self._update_due <= 0):
            self._update_due = self.update_speed
            return True
        else:
            self._update_due -= update_speed
            return False

    # Update the meter by adjusting krell size and updating the value text
    def update(self):
        """
        Update the krell and text on the meter to most recent values
        """
        try:
            __krell_value, __text_value = self.getMeterValues()
        except Exception, e:
            print e
            __krell_value = [0.0,]*self._number_of_meters
            __text_value = ["read failure",]*self._number_of_meters
        
        self.value_text = __text_value
        
        for index in range(self._number_of_meters):
            if (__krell_value[index] == self._last_value[index]):
                continue
                
            adesklets.set(self._macro_krell[index], int(self._krell_width * __krell_value[index]))
            adesklets.set(self._macro_text[index], self.base_text[index] + " " + self.value_text[index])
            
            adesklets.play(*self._macro[index])

            self._last_value[index] = __krell_value[index]
        
    def getWidth(self):
        """
        Get the width of the meter
        """
#        assert self._bounds[2] - self._bounds[0] >= 0
        return(self._bounds[2] - self._bounds[0])
        
    def getHeight(self):
        """
        Get the height of the meter
        """
 #       assert self._bounds[3] - self._bounds[1] >= 0
        return(self._bounds[3] - self._bounds[1])

# We only need to gra the cpu load, and return it
class CPUMeter (AbstractMeter):
    
    def create(self, location, basedir, config_dictionary):
        assert type(config_dictionary) is dict
        
        config_dictionary["base text"] = ("CPU Load",)
        config_dictionary["value text length"] = (4,)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)    
        
    def getMeterValues(self):
        cpu_stats = statgrab.sg_get_cpu_percents()
        krell_value = (100 - cpu_stats["idle"])/100.0
        text_value = str(int(100 * krell_value)) + "%"
        
#        assert type(krell_value) is float and \
#            krell_value >= 0 and krell_value <= 1
#        assert len(text_value) > 1 and len(text_value) <= 12
        
        return [krell_value,], [text_value,]
        

# Just grab memory stats and return them
class MemoryMeter (AbstractMeter):
    
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
        
        config_dictionary["base text"] = ("Mem",)
        config_dictionary["value text length"] = (10,)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
            
    
    def getMeterValues(self):
        mem_stats = statgrab.sg_get_mem_stats()
        krell_value = float(mem_stats["total"] - mem_stats["free"] - 
                          mem_stats["cache"])/float(mem_stats["total"])
        text_value = str(int((mem_stats["free"] + mem_stats["cache"])/(1024.0**2))) + "M Free"
        
#        assert type(krell_value) is float and \
#            krell_value >= 0 and krell_value <= 1
#        assert len(text_value) > 1 and len(text_value) <= 12
        
        return [krell_value,], [text_value,]

class SwapMeter (AbstractMeter):
    
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
        
        config_dictionary["base text"] = ("Swap",)
        config_dictionary["value text length"] = (12,)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
        
    def getMeterValues(self):
        swap_stats = statgrab.sg_get_swap_stats()
        krell_value = float(swap_stats["total"] - swap_stats["free"])/float(swap_stats["total"])
        text_value = str(int(swap_stats["free"]/(1024.0**2))) + "M Free"
        
#        assert type(krell_value) is float and \
#            krell_value >= 0 and krell_value <= 1
#        assert len(text_value) > 1 and len(text_value) <= 12
        
        return [krell_value,], [text_value,]        

class ProcessMeter (AbstractMeter):
    
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
#        assert "max processes" in config_dictionary.keys() and\
#            type(config_dictionary["max processes"]) is int

        self.max_processes = config_dictionary["max processes"]

        config_dictionary["base text"] = ("Processes",)
        config_dictionary["value text length"] = (12,)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
        
    def getMeterValues(self):
        proc_stats = statgrab.sg_get_process_count()
        krell_value = log(float(proc_stats["total"]))/log(float(self.max_processes))
        text_value = str(proc_stats["total"]) + " procs"

#        assert type(krell_value) is float and \
#            krell_value >= 0 and krell_value <= 1
#        assert len(text_value) > 1 and len(text_value) <= 12
        
        return [krell_value,], [text_value,]

# We expect 2 meters, 1 for receieved, one for sent
# We also need an interface name (eth0, ppp0, etc), and maximum upload and download speeds        
class NetworkMeter (AbstractMeter):
    
    def __init__(self):
        AbstractMeter.__init__(self)
        self.interface_name = None
        self.max_down_speed = None
        self.max_up_speed = None
        
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
#        assert "interface name" in config_dictionary.keys()
#        assert "max up speed" in config_dictionary.keys() and \
#            type(config_dictionary["max up speed"]) is int
#        assert "max down speed" in config_dictionary.keys() and \
#            type(config_dictionary["max down speed"]) is int
 
        self.interface_name = config_dictionary["interface name"]
        self.max_down_speed = config_dictionary["max down speed"]
        self.max_up_speed = config_dictionary["max up speed"]
        
        config_dictionary["base text"] = (self.interface_name + " down", 
                                          self.interface_name + " up")
        config_dictionary["value text length"] = (8,8)
        config_dictionary["number of meters"] = 2
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
 
    
    def getMeterValues(self):
        net_stats = statgrab.sg_get_network_io_stats_diff()
        
        krell_value = [0.0,0.0]
        text_value = ["interface not found", "interface not found"]
        
        for interface_stats in net_stats:
            if (interface_stats["interface_name"] == self.interface_name):
                
                tmp_down = max(float(interface_stats["rx"])/
                               (max(interface_stats["systime"], 1) * 1024.0), 1)
                tmp_up = max(float(interface_stats["tx"])/
                             (max(interface_stats["systime"], 1) * 1024.0), 1)
                
                # Use a log scale so that the krell actually moves
                krell_value = [min(log(tmp_down)/log(self.max_down_speed), 1),
                               min(log(tmp_up)/log(self.max_up_speed), 1)]
                text_value = [str(int((interface_stats["rx"])/
                                  (1024.0 * max(interface_stats["systime"], 1)))) + "k/s", 
                              str(int((interface_stats["tx"])/
                                  (1024.0 * max(interface_stats["systime"], 1)))) + "k/s"]
                              
#        assert type(krell_value) is list and \
#            type(krell_value[0]) is float and type(krell_value[1]) is float
#        assert type(text_value) is list and \
#            type(text_value[0]) is str and type(text_value[1]) is str
#        assert krell_value[0] >= 0 and krell_value[0] <= 1
#        assert krell_value[1] >= 0 and krell_value[1] <= 1
                              
        return krell_value, text_value

# Pretty muc identical to network io, except we need the disk name instead of interface name
# and we need the max read/write speeds instead of up and down speeds        
class DiskIOMeter (AbstractMeter):
    
    def __init__(self):
        AbstractMeter.__init__(self)
        self.disk_name = None
        self.max_read_speed = None
        self.max_write_speed = None
        
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
#        assert "disk name" in config_dictionary.keys()
#        assert "max read speed" in config_dictionary.keys() and \
#            type(config_dictionary["max read speed"]) is int
#        assert "max write speed" in config_dictionary.keys() and \
#            type(config_dictionary["max write speed"]) is int
        
        self.disk_name = config_dictionary["disk name"]
        self.max_read_speed = config_dictionary["max read speed"]
        self.max_write_speed = config_dictionary["max write speed"]
        
        config_dictionary["base text"] = (self.disk_name + " read", self.disk_name + " write")
        config_dictionary["value text length"] = (8,8)
        config_dictionary["number of meters"] = 2
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
    
    def getMeterValues(self):
        io_stats = statgrab.sg_get_disk_io_stats_diff()
        
        krell_value = [0.0,0.0]
        text_value = ["disk not found", "disk not found"]
                
        for disk_stats in io_stats:
            if (disk_stats["disk_name"] == self.disk_name):
                
                tmp_read = max(float(disk_stats["read_bytes"])/
                               (max(disk_stats["systime"], 1) * 1024.0), 1)
                tmp_write = max(float(disk_stats["write_bytes"])/
                                (max(disk_stats["systime"], 1) * 1024.0), 1)
                
                # Use a log scale so that the krell actually moves
                krell_value = [min(log(tmp_read)/log(self.max_read_speed), 1),
                               min(log(tmp_write)/log(self.max_write_speed), 1)]
                text_value = [str(int((disk_stats["read_bytes"])/
                                 (1024.0 * max(disk_stats["systime"], 1)))) + "k/s", 
                              str(int((disk_stats["write_bytes"])/
                                  (1024.0 * max(disk_stats["systime"], 1)))) + "k/s"]
                              
#        assert type(krell_value) is list and \
#            type(krell_value[0]) is float and type(krell_value[1]) is float
#        assert type(text_value) is list and \
#            type(text_value[0]) is str and type(text_value[1]) is str
#        assert krell_value[0] >= 0 and krell_value[0] <= 1
#        assert krell_value[1] >= 0 and krell_value[1] <= 1
                              
        return krell_value, text_value    

# Use a meter for each fixed mount point - so we need to gather that information
# at creation time and build it into the configuration dictionary
class DiskSpaceMeter (AbstractMeter):
    """
    Create individual meters for mountpoints and display how much
    free space is available on each mount point
    """
    
    def create(self, location, basedir, config_dictionary):
#        assert type(config_dictionary) is dict
        
        config_update = {}
        fs_stats = statgrab.sg_get_fs_stats()
        
        # find number of meters required
        config_update["number of meters"] = len(fs_stats)
        
        # get names of all the mount points for base text
        mount_points = []
        for partition_stats in fs_stats:
            mount_points.append(partition_stats["mnt_point"])
            
        config_update["base text"] = tuple(mount_points)
        config_update["value text length"] = (12,) * config_update["number of meters"]
        
        config_dictionary.update(config_update)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
        
    def getMeterValues(self):
        
        fs_stats = statgrab.sg_get_fs_stats()
        
        krell_value = []
        text_value = []
        
        for partition_stats in fs_stats:
            if (partition_stats["size"] != 0):
                krell_value.append(float(partition_stats["used"])/partition_stats["size"])
                text_value.append(str(int(partition_stats["avail"]/(1024.0**2))) + "M Free")
            else:
                krell_value.append(float(0))
                text_value.append(str(int(0)) + "M Free") 
        
#        assert type(krell_value) is list and \
#            type(krell_value[0]) is float and type(krell_value[1]) is float
#        assert type(text_value) is list and \
#            type(text_value[0]) is str and type(text_value[1]) is str
#        assert krell_value[0] >= 0 and krell_value[0] <= 1
#        assert krell_value[1] >= 0 and krell_value[1] <= 1
                              
        return krell_value, text_value        

# Create a base class for all meters that read data from ACPI
# files in /proc/acpi - provide parsing facilities.
class ACPIMeter (AbstractMeter):
    """Abstract ACPI meter for all ACPI related monitoring"""
    
    def create(self, location, basedir, config_dictionary):
        self.file = config_dictionary["file"]
        AbstractMeter.create(self, location, basedir, config_dictionary)
    
    def parseFile(self, file):
        acpi_data = {}
        acpi_file = open(file, "r")
        acpi_lines = acpi_file.readlines()
        acpi_file.close()
        acpi_lines = map(strip, acpi_lines)
        
        for line in acpi_lines:
            [key, value] = split(":\s+", line)
            acpi_data[key] = value
            
        return acpi_data

# Read battery information and display a meter showing charge        
class BatteryMeter (ACPIMeter):
    """Meter battery charge"""
    
    def create(self, location, basedir, config_dictionary):
        self.state_file = config_dictionary["state_file"]
        self.info_file = config_dictionary["info_file"]
        
        info = self.parseFile(self.info_file)
        
        self.capacity = float(info["design capacity"].split()[0])
        
        config_dictionary["base text"] = ("Battery "+config_dictionary["label"],)
        config_dictionary["value text length"] = (12,)
        
        AbstractMeter.create(self, location, basedir, config_dictionary)
    
    def getMeterValues(self):
        state = self.parseFile(self.state_file)
        
        if (state["present"] != "yes"):
            return [0.0,], ["not present",]
            
        text_value = state["charging state"]
        krell_value = float(state["remaining capacity"].split()[0])/self.capacity
        
        return [krell_value,], [text_value,]
        
# get CPU temperature information and display in a meter        
class TemperatureMeter (ACPIMeter):
    
    def create(self, location, basedir, config_dictionary):

        config_dictionary["base text"] = ("Temperature",)
        config_dictionary["value text length"] = (6,)
        
        self.max_temp = config_dictionary["max_temp"]
        
        ACPIMeter.create(self, location, basedir, config_dictionary)
        
    def getMeterValues(self):
        data = self.parseFile(self.file)
        
        text_value = data["temperature"]
        krell_value = float(data["temperature"].split()[0])/self.max_temp
        
        return [krell_value,], [text_value,]
        
        
class EventHandler (adesklets.Events_handler):
    """
    Inherited from adesklets, this handles the signals to draw to screen and update
    """
    
    def __init__(self, basedir):
        if len(basedir) == 0:
            self.basedir = '.'
        else:
            self.basedir = basedir
            
        self.meters = []
        self.meter_padding = 16
        self.update_speed = 1
                    
        adesklets.Events_handler.__init__(self)

    def __del__(self):
        """
        Clean up on deletion
        """
        adesklets.Events_handler.__del__(self)
                
    def ready(self):
        """
        Actual instatiation routine
        """
        self.id = adesklets.get_id()
        
        self.config=Config(adesklets.get_id(),
                           join(self.basedir,'config.txt'))
                           
        self.update_speed = self.config["update speed"]
        
        if (self.config.has_key("theme")):
            self._apply_theme(self.config["theme"])
                           
        # Adjust height for rounded border corners
        tmp_height = self.meter_padding + 16
        tmp_width = 0
                           
        for meter in self.config["meters"]:
            meter[1]["background colour"] = self.config["background colour"]
            meter[1]["text colour"] = self.config["text colour"]
            self.meters.append(eval(meter[0] + "()"))
            self.meters[-1].create((8,tmp_height), self.basedir, meter[1])
            tmp_height += self.meters[-1].getHeight() + self.meter_padding
            tmp_width = max(tmp_width, self.meters[-1].getWidth())
            
        # Adjust width for rounded border corners
        self.width = tmp_width + 34
        self.height = tmp_height + self.meter_padding
        
        self.buffer=adesklets.create_image(self.width, self.height)
        adesklets.window_resize(self.width, self.height)
                                
        adesklets.window_set_transparency(True)
        adesklets.menu_add_separator()
        adesklets.menu_add_item('Configure')
        adesklets.window_show()
        self.run_display()
        
    def _apply_theme(self, theme_filename):
        theme_file = open(theme_filename, "r")
        theme_data = theme_file.read(-1)
        # strip comments
        theme_data = sub("\n#.*?\n", "\n", theme_data)
        theme_dictionary = eval(theme_data)
        
        for meter in self.config["meters"]:
            meter[1].update(theme_dictionary[meter[0]])
            
        self.config["background images"] = theme_dictionary["background images"]
        self.config["background colour"] = theme_dictionary["background colour"]
        self.config["text colour"] = theme_dictionary["text colour"]
        
        self.basedir = dirname(theme_filename)
        if (len(self.basedir) == 0):
            self.basedir = '.'
        
    def run_display(self):
        """
        Build the base display and meters
        """
        # Create buffer and add base/background image
        adesklets.context_set_image(0)
        adesklets.context_set_color(0,0,0,0)
        adesklets.context_set_blend(False)
        adesklets.image_fill_rectangle(0,0,self.width, self.height)
        
        bg_top = adesklets.load_image(join(self.basedir,self.config["background images"][0]))
        adesklets.context_set_image(bg_top)
        top_width = adesklets.image_get_width()
        top_height = adesklets.image_get_height() 
        adesklets.context_set_image(0)
        
        adesklets.blend_image_onto_image(bg_top,1,
                                         0,0,top_width,top_height,
                                         0,0,self.width,top_height)
        adesklets.free_image(bg_top)
        
        bg_bottom = adesklets.load_image(join(self.basedir,self.config["background images"][2]))
        adesklets.context_set_image(bg_bottom)
        bot_width = adesklets.image_get_width()
        bot_height = adesklets.image_get_height() 
        adesklets.context_set_image(0)
             
        adesklets.blend_image_onto_image(bg_bottom,1,
                                         0,0,bot_width,bot_height,
                                         0,self.height-bot_height,self.width,bot_height)
        adesklets.free_image(bg_bottom)

        bg_middle = adesklets.load_image(join(self.basedir,self.config["background images"][1]))
        adesklets.context_set_image(bg_middle)
        mid_width = adesklets.image_get_width()
        mid_height = adesklets.image_get_height() 
        adesklets.context_set_image(0)
        
        adesklets.blend_image_onto_image(bg_middle,1,
                                         0,0,mid_width,mid_height,
                                         0,top_height,self.width,self.height-(bot_height + top_height))
        adesklets.free_image(bg_middle)
        
        for meter in self.meters:
            meter.display()
            
    def menu_fire(self, delayed, menu_id, item):
        """
        A menu event has occured
        """
        if item=='Configure':
            editor=getenv('EDITOR')
            if editor:
                system('xterm -e %s %s/config.txt &' % (editor, self.basedir))

    def alarm(self):
        """
        An update has been called for
        """
        self.block()

        for meter in self.meters:
            if (meter.requireUpdate(self.update_speed)):
                meter.update()

        self.unblock()
        return self.update_speed
                
EventHandler(dirname(__file__)).pause()
